@@ -2,6 +2,7 @@ package helpers
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"io"
 	"net/http"
@@ -36,6 +37,7 @@ type CacheArchiverCommand struct {
 	GoCloudURL             string   `long:"gocloud-url" description:"Go Cloud URL of remote cache resource (requires credentials)"`
 	Timeout                int      `long:"timeout" description:"Overall timeout for cache uploading request (in minutes)"`
 	Headers                []string `long:"header" description:"HTTP headers to send with PUT request (in form of 'key:value')"`
+	Metadata               metadata `long:"metadata" env:"CACHE_METADATA" description:"Metadata for the cache artifact (JSON encoded key-value-pairs, e.g. '{\"foo\":\"bar\",\"blerp\":\"blip\"}')"`
 	CompressionLevel       string   `long:"compression-level" env:"CACHE_COMPRESSION_LEVEL" description:"Compression level (fastest, fast, default, slow, slowest)"`
 	CompressionFormat      string   `long:"compression-format" env:"CACHE_COMPRESSION_FORMAT" description:"Compression format (zip, tarzstd)"`
 	MaxUploadedArchiveSize int64    `long:"max-uploaded-archive-size" env:"CACHE_MAX_UPLOADED_ARCHIVE_SIZE" description:"Limit the size of the cache archive being uploaded to cloud storage, in bytes."`
@@ -45,6 +47,12 @@ type CacheArchiverCommand struct {
 	mux    *blob.URLMux
 }
 
+type metadata map[string]string
+
+func (m *metadata) UnmarshalFlag(raw string) error {
+	return json.Unmarshal([]byte(raw), m)
+}
+
 func (c *CacheArchiverCommand) getClient() *CacheClient {
 	if c.client == nil {
 		c.client = NewCacheClient(c.Timeout)
@@ -130,7 +138,11 @@ func (c *CacheArchiverCommand) handleGoCloudURL(file io.Reader) error {
 	}
 	defer b.Close()
 
-	writer, err := b.NewWriter(ctx, objectName, nil)
+	opts := &blob.WriterOptions{
+		Metadata: c.Metadata,
+	}
+
+	writer, err := b.NewWriter(ctx, objectName, opts)
 	if err != nil {
 		return err
 	}
@@ -220,6 +232,11 @@ func (c *CacheArchiverCommand) Execute(*cli.Context) {
 		logrus.Fatalln(err)
 	}
 
+	err = writeCacheMetadataFile(c.File, c.Metadata)
+	if err != nil {
+		logrus.Fatalln(err)
+	}
+
 	c.uploadArchiveIfNeeded(size)
 }
 
@@ -266,16 +283,8 @@ func (c *CacheArchiverCommand) uploadArchiveIfNeeded(size int64) {
 }
 
 func (c *CacheArchiverCommand) setHeaders(req *http.Request, fi os.FileInfo) {
-	if len(c.Headers) > 0 {
-		for _, header := range c.Headers {
-			parsed := strings.SplitN(header, ":", 2)
-
-			if len(parsed) != 2 {
-				continue
-			}
-
-			req.Header.Set(strings.TrimSpace(parsed[0]), strings.TrimSpace(parsed[1]))
-		}
+	for k, v := range split(c.Headers) {
+		req.Header.Set(strings.TrimSpace(k), strings.TrimSpace(v))
 	}
 
 	// Set default headers. But don't override custom Content-Type.
@@ -285,8 +294,24 @@ func (c *CacheArchiverCommand) setHeaders(req *http.Request, fi os.FileInfo) {
 	req.Header.Set("Last-Modified", fi.ModTime().UTC().Format(http.TimeFormat))
 }
 
+func split(raw []string) map[string]string {
+	const sep = ":"
+
+	data := make(map[string]string, len(raw))
+
+	for _, s := range raw {
+		k, v, ok := strings.Cut(s, sep)
+		if !ok {
+			continue
+		}
+		data[k] = v
+	}
+
+	return data
+}
+
 func init() {
-	common.RegisterCommand2(
+	common.RegisterCommand(
 		"cache-archiver",
 		"create and upload cache artifacts (internal)",
 		&CacheArchiverCommand{
