@@ -206,6 +206,7 @@ type FeaturesInfo struct {
 	ServiceExecutorOpts     bool `json:"service_executor_opts"`
 	CancelGracefully        bool `json:"cancel_gracefully"`
 	NativeStepsIntegration  bool `json:"native_steps_integration"`
+	TwoPhaseJobCommit       bool `json:"two_phase_job_commit"`
 }
 
 type ConfigInfo struct {
@@ -225,8 +226,8 @@ type RegisterRunnerParameters struct {
 
 type RegisterRunnerRequest struct {
 	RegisterRunnerParameters
-	Info  VersionInfo `json:"info,omitempty"`
-	Token string      `json:"token,omitempty"`
+	Info  Info   `json:"info,omitempty"`
+	Token string `json:"token,omitempty"`
 }
 
 type RegisterRunnerResponse struct {
@@ -265,7 +266,7 @@ type ResetTokenResponse struct {
 	TokenExpiresAt  time.Time `json:"token_expires_at,omitempty"`
 }
 
-type VersionInfo struct {
+type Info struct {
 	Name         string       `json:"name,omitempty"`
 	Version      string       `json:"version,omitempty"`
 	Revision     string       `json:"revision,omitempty"`
@@ -275,10 +276,11 @@ type VersionInfo struct {
 	Shell        string       `json:"shell,omitempty"`
 	Features     FeaturesInfo `json:"features"`
 	Config       ConfigInfo   `json:"config,omitempty"`
+	Labels       Labels       `json:"labels,omitempty"`
 }
 
 type JobRequest struct {
-	Info       VersionInfo  `json:"info,omitempty"`
+	Info       Info         `json:"info,omitempty"`
 	Token      string       `json:"token,omitempty"`
 	SystemID   string       `json:"system_id,omitempty"`
 	LastUpdate string       `json:"last_update,omitempty"`
@@ -292,10 +294,22 @@ type SessionInfo struct {
 }
 
 type JobInfo struct {
-	Name        string `json:"name"`
-	Stage       string `json:"stage"`
-	ProjectID   int64  `json:"project_id"`
-	ProjectName string `json:"project_name"`
+	Name  string `json:"name"`
+	Stage string `json:"stage"`
+
+	ProjectID       int64  `json:"project_id"`
+	ProjectName     string `json:"project_name"`
+	ProjectFullPath string `json:"project_full_path"`
+
+	NamespaceID     int64 `json:"namespace_id"`
+	RootNamespaceID int64 `json:"root_namespace_id"`
+	OrganizationID  int64 `json:"organization_id"`
+
+	InstanceID   string `json:"instance_id"`
+	InstanceUUID string `json:"instance_uuid"`
+
+	UserID       int64  `json:"user_id"`
+	ScopedUserID *int64 `json:"scoped_user_id,omitempty"`
 
 	TimeInQueueSeconds                       float64 `json:"time_in_queue_seconds"`
 	ProjectJobsRunningOnInstanceRunnersCount string  `json:"project_jobs_running_on_instance_runners_count"`
@@ -1099,7 +1113,7 @@ func (j *JobResponse) UnsupportedOptions() error {
 }
 
 type UpdateJobRequest struct {
-	Info          VersionInfo      `json:"info,omitempty"`
+	Info          Info             `json:"info,omitempty"`
 	Token         string           `json:"token,omitempty"`
 	State         JobState         `json:"state,omitempty"`
 	FailureReason JobFailureReason `json:"failure_reason,omitempty"`
@@ -1170,6 +1184,7 @@ type JobTrace interface {
 	io.Writer
 	Success() error
 	Fail(err error, failureData JobFailureData) error
+	Finish()
 	SetCancelFunc(cancelFunc context.CancelFunc)
 	Cancel() bool
 	SetAbortFunc(abortFunc context.CancelFunc)
