@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"io"
 	"net"
+	"strings"
 	"time"
 
 	"github.com/sirupsen/logrus"
@@ -68,7 +69,7 @@ func (ref *acquisitionRef) Prepare(
 
 	info, err := ref.acq.InstanceConnectInfo(dialCtx)
 	if cause := context.Cause(dialCtx); cause != nil {
-		return nil, &common.BuildError{Inner: cause, FailureReason: reasonFromCause(cause)}
+		return nil, &common.BuildError{Inner: cause, FailureReason: common.RunnerSystemFailure}
 	}
 	if err != nil {
 		return nil, fmt.Errorf("getting instance connect info: %w", err)
@@ -91,12 +92,19 @@ func (ref *acquisitionRef) Prepare(
 		UseExternalAddr: useExternalAddr,
 	}
 
-	logger.Println(fmt.Sprintf("Dialing instance %s...", info.ID))
-	fleetingDialer, err := ref.dialAcquisitionInstance(dialCtx, info, fleetingDialOpts)
-	if cause := context.Cause(dialCtx); cause != nil {
-		return nil, &common.BuildError{Inner: cause, FailureReason: reasonFromCause(cause)}
+	ident := []string{info.ID}
+	if options.Config.Autoscaler.LogInternalIP && info.InternalAddr != "" {
+		ident = append(ident, info.InternalAddr)
+	}
+	if options.Config.Autoscaler.LogExternalIP && info.ExternalAddr != "" {
+		ident = append(ident, info.ExternalAddr)
 	}
 
+	logger.Println(fmt.Sprintf("Dialing instance %s...", strings.Join(ident, ", ")))
+	fleetingDialer, err := ref.dialAcquisitionInstance(dialCtx, info, fleetingDialOpts)
+	if cause := context.Cause(dialCtx); cause != nil {
+		return nil, &common.BuildError{Inner: cause, FailureReason: common.RunnerSystemFailure}
+	}
 	if err != nil {
 		return nil, err
 	}
@@ -128,17 +136,6 @@ func (ref *acquisitionRef) Prepare(
 	return client, nil
 }
 
-func reasonFromCause(cause error) common.JobFailureReason {
-	switch {
-	case errors.Is(cause, context.DeadlineExceeded):
-		return common.JobExecutionTimeout
-	case errors.Is(cause, context.Canceled):
-		return common.JobCanceled
-	default:
-		return common.RunnerSystemFailure
-	}
-}
-
 func (ref *acquisitionRef) WithContext(ctx context.Context) (context.Context, context.CancelFunc) {
 	if ref.acq == nil {
 		return context.WithCancel(ctx)
@@ -263,7 +260,7 @@ func (ref *acquisitionRef) createTunneledDialer(
 
 	client, err := ref.dialTunnel(ctx, info, options)
 	if cause := context.Cause(ctx); cause != nil {
-		return nil, &common.BuildError{Inner: cause, FailureReason: reasonFromCause(cause)}
+		return nil, &common.BuildError{Inner: cause, FailureReason: common.RunnerSystemFailure}
 	}
 
 	return client, err
